@@ -523,6 +523,7 @@ local function _automatic_fetch(premature, self)
     end
 
     if not (health_check.conf and health_check.conf.shm_name) then
+        -- used for worker processes to synchronize configuration
         local _, err = health_check.init({
             shm_name = health_check_shm_name,
             fail_timeout = self.health_check_timeout,
@@ -811,13 +812,12 @@ function _M.init()
         return true
     end
 
-    local etcd_cli, err = get_etcd()
+    -- don't go through proxy during start because the proxy is not available
+    local etcd_cli, prefix, err = etcd_apisix.new_without_proxy()
     if not etcd_cli then
         return nil, "failed to start a etcd instance: " .. err
     end
 
-    local etcd_conf = local_conf.etcd
-    local prefix = etcd_conf.prefix
     local res, err = readdir(etcd_cli, prefix, create_formatter(prefix))
     if not res then
         return nil, err
