@@ -19,26 +19,30 @@
 --
 -- @module core.etcd
 
-local fetch_local_conf = require("apisix.core.config_local").local_conf
-local array_mt         = require("apisix.core.json").array_mt
-local etcd             = require("resty.etcd")
-local clone_tab        = require("table.clone")
-local health_check     = require("resty.etcd.health_check")
-local ipairs           = ipairs
-local setmetatable     = setmetatable
-local string           = string
-local tonumber         = tonumber
+local fetch_local_conf  = require("apisix.core.config_local").local_conf
+local array_mt          = require("apisix.core.json").array_mt
+local etcd              = require("resty.etcd")
+local clone_tab         = require("table.clone")
+local health_check      = require("resty.etcd.health_check")
+local ipairs            = ipairs
+local setmetatable      = setmetatable
+local string            = string
+local tonumber          = tonumber
+local ngx_config_prefix = ngx.config.prefix()
+local ngx_socket_tcp    = ngx.socket.tcp
+
+
+local is_http = ngx.config.subsystem == "http"
 local _M = {}
 
 
--- this function create the etcd client instance used in the Admin API
-local function new()
-    local local_conf, err = fetch_local_conf()
-    if not local_conf then
-        return nil, nil, err
-    end
+local function has_mtls_support()
+    local s = ngx_socket_tcp()
+    return s.tlshandshake ~= nil
+end
 
-    local etcd_conf = clone_tab(local_conf.etcd)
+
+local function _new(etcd_conf)
     local prefix = etcd_conf.prefix
     etcd_conf.http_host = etcd_conf.host
     etcd_conf.host = nil
@@ -63,25 +67,106 @@ local function new()
         end
     end
 
-    -- enable etcd health check retry for curr worker
-    if not health_check.conf then
-        health_check.init({
-            max_fails = #etcd_conf.http_host,
-            retry = true,
-        })
-    end
-
-    local etcd_cli
-    etcd_cli, err = etcd.new(etcd_conf)
+    local etcd_cli, err = etcd.new(etcd_conf)
     if not etcd_cli then
         return nil, nil, err
     end
 
     return etcd_cli, prefix
 end
+
+
+local function new()
+    local local_conf, err = fetch_local_conf()
+    if not local_conf then
+        return nil, nil, err
+    end
+
+    local etcd_conf = clone_tab(local_conf.etcd)
+    local proxy_by_conf_server = false
+
+    if local_conf.deployment then
+        if local_conf.deployment.role == "traditional"
+            -- we proxy the etcd requests in traditional mode so we can test the CP's behavior in
+            -- daily development. However, a stream proxy can't be the CP.
+            -- Hence, generate a HTTP conf server to proxy etcd requests in stream proxy is
+            -- unnecessary and inefficient.
+            and is_http
+        then
+            local sock_prefix = ngx_config_prefix
+            etcd_conf.unix_socket_proxy =
+                "unix:" .. sock_prefix .. "/conf/config_listen.sock"
+            etcd_conf.host = {"http://127.0.0.1:2379"}
+            proxy_by_conf_server = true
+
+        elseif local_conf.deployment.role == "control_plane" then
+            local addr = local_conf.deployment.role_control_plane.conf_server.listen
+            etcd_conf.host = {"https://" .. addr}
+            etcd_conf.tls = {
+                verify = false,
+            }
+
+            if has_mtls_support() and local_conf.deployment.certs.cert then
+                local cert = local_conf.deployment.certs.cert
+                local cert_key = local_conf.deployment.certs.cert_key
+                etcd_conf.tls.cert = cert
+                etcd_conf.tls.key = cert_key
+            end
+
+            proxy_by_conf_server = true
+
+        elseif local_conf.deployment.role == "data_plane" then
+            if has_mtls_support() and local_conf.deployment.certs.cert then
+                local cert = local_conf.deployment.certs.cert
+                local cert_key = local_conf.deployment.certs.cert_key
+
+                if not etcd_conf.tls then
+                    etcd_conf.tls = {}
+                end
+
+                etcd_conf.tls.cert = cert
+                etcd_conf.tls.key = cert_key
+            end
+        end
+    end
+
+    -- if an unhealthy etcd node is selected in a single admin read/write etcd operation,
+    -- the retry mechanism for health check can select another healthy etcd node
+    -- to complete the read/write etcd operation.
+    if proxy_by_conf_server then
+        -- health check is done in conf server
+        health_check.disable()
+    elseif not health_check.conf then
+        health_check.init({
+            max_fails = 1,
+            retry = true,
+        })
+    end
+
+    return _new(etcd_conf)
+end
 _M.new = new
 
 
+---
+-- Create an etcd client which will connect to etcd without being proxyed by conf server.
+-- This method is used in init_worker phase when the conf server is not ready.
+--
+-- @function core.etcd.new_without_proxy
+-- @treturn table|nil the etcd client, or nil if failed.
+-- @treturn string|nil the configured prefix of etcd keys, or nil if failed.
+-- @treturn nil|string the error message.
+function _M.new_without_proxy()
+    local local_conf, err = fetch_local_conf()
+    if not local_conf then
+        return nil, nil, err
+    end
+
+    local etcd_conf = clone_tab(local_conf.etcd)
+    return _new(etcd_conf)
+end
+
+
 -- convert ETCD v3 entry to v2 one
 local function kvs_to_node(kvs)
     local node = {}
@@ -234,6 +319,10 @@ local function set(key, value, ttl)
         return nil, err
     end
 
+    if res.body.error then
+        return nil, res.body.error
+    end
+
     res.headers["X-Etcd-Index"] = res.body.header.revision
 
     -- etcd v3 set would not return kv info
@@ -382,7 +471,7 @@ end
 -- --   etcdserver = "3.5.0"
 -- -- }
 function _M.server_version()
-    local etcd_cli, err = new()
+    local etcd_cli, _, err = new()
     if not etcd_cli then
         return nil, err
     end
