@@ -180,15 +180,10 @@ local function send_to_skywalking(log_message)
     httpc:set_timeout(config.timeout * 1000)
 
     local entries = {}
-    local service_instance_name = config.skywalking.service_instance_name
-    if service_instance_name == "$hostname" then
-        service_instance_name = core.utils.gethostname()
-    end
-
     for i = 1, #log_message, 2 do
         local content = {
             service = config.skywalking.service_name,
-            serviceInstance = service_instance_name,
+            serviceInstance = config.skywalking.service_instance_name,
             endpoint = "",
             body = {
                 text = {
