@@ -20,6 +20,8 @@ local json              = core.json
 local pb                = require("pb")
 local ngx               = ngx
 local string            = string
+local table             = table
+local ipairs            = ipairs
 local tonumber          = tonumber
 local type              = type
 
@@ -51,6 +53,41 @@ function _M.find_method(proto, service, method)
 end
 
 
+function _M.set_options(proto, options)
+    local cur_opts = proto.options
+    if cur_opts then
+        if cur_opts == options then
+            -- same route
+            return
+        end
+
+        local same = true
+        table.sort(options)
+        for i, v in ipairs(options) do
+            if cur_opts[i] ~= v then
+                same = false
+                break
+            end
+        end
+
+        if same then
+            -- Routes have the same configuration, usually the default one.
+            -- As this is a small optimization, we don't care about routes have different
+            -- configuration but have the same effect eventually.
+            return
+        end
+    else
+        table.sort(options)
+    end
+
+    for _, opt in ipairs(options) do
+        pb.option(opt)
+    end
+
+    proto.options = options
+end
+
+
 local function get_request_table()
     local method = ngx.req.get_method()
     local content_type = ngx.req.get_headers()["Content-Type"] or ""
@@ -110,6 +147,22 @@ function _M.map_message(field, default_values, request_table)
         if ty ~= "enum" and field_type:sub(1, 1) == "." then
             if request_table[name] == nil then
                 sub = default_values and default_values[name]
+            elseif core.table.isarray(request_table[name]) then
+                local sub_array = core.table.new(#request_table[name], 0)
+                for i, value in ipairs(request_table[name]) do
+                    local sub_array_obj
+                    if type(value) == "table" then
+                        sub_array_obj, err = _M.map_message(field_type,
+                                default_values and default_values[name], value)
+                        if err then
+                            return nil, err
+                        end
+                    else
+                        sub_array_obj = value
+                    end
+                    sub_array[i] = sub_array_obj
+                end
+                sub = sub_array
             else
                 sub, err = _M.map_message(field_type, default_values and default_values[name],
                                           request_table[name])
